@@ -101,6 +101,8 @@ const (
 	Report         PDUType = 0xa8 // v3
 )
 
+//go:generate stringer -type=PDUType
+
 // SNMPv3: User-based Security Model Report PDUs and
 // error types as per https://tools.ietf.org/html/rfc3414
 const (
@@ -154,6 +156,31 @@ func NewLogger(logger LoggerInterface) Logger {
 	}
 }
 
+func (packet *SnmpPacket) SafeString() string {
+	sp := ""
+	if packet.SecurityParameters != nil {
+		sp = packet.SecurityParameters.SafeString()
+	}
+	return fmt.Sprintf("Version:%s, MsgFlags:%s, SecurityModel:%s, SecurityParameters:%s, ContextEngineID:%s, ContextName:%s, Community:%s, PDUType:%s, MsgID:%d, RequestID:%d, MsgMaxSize:%d, Error:%s, ErrorIndex:%d, NonRepeaters:%d, MaxRepetitions:%d, Variables:%v",
+		packet.Version,
+		packet.MsgFlags,
+		packet.SecurityModel,
+		sp,
+		packet.ContextEngineID,
+		packet.ContextName,
+		packet.Community,
+		packet.PDUType,
+		packet.MsgID,
+		packet.RequestID,
+		packet.MsgMaxSize,
+		packet.Error,
+		packet.ErrorIndex,
+		packet.NonRepeaters,
+		packet.MaxRepetitions,
+		packet.Variables,
+	)
+}
+
 // GoSNMP
 // send/receive one snmp request
 func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
@@ -238,7 +265,7 @@ func (x *GoSNMP) sendOneRequest(packetOut *SnmpPacket,
 		if x.PreSend != nil {
 			x.PreSend(x)
 		}
-		x.Logger.Printf("SENDING PACKET: %#+v", *packetOut)
+		x.Logger.Printf("SENDING PACKET: %s", packetOut.SafeString())
 		// If using UDP and unconnected socket, send packet directly to stored address.
 		if uconn, ok := x.Conn.(net.PacketConn); ok && x.uaddr != nil {
 			_, err = uconn.WriteTo(outBuf, x.uaddr)
@@ -437,7 +464,7 @@ func (x *GoSNMP) send(packetOut *SnmpPacket, wait bool) (result *SnmpPacket, err
 	}
 
 	if result.Version == Version3 {
-		x.Logger.Printf("SEND STORE SECURITY PARAMS from result: %+v", result)
+		x.Logger.Printf("SEND STORE SECURITY PARAMS from result: %s", result.SecurityParameters.SafeString())
 		err = x.storeSecurityParameters(result)
 
 		if result.PDUType == Report && len(result.Variables) == 1 {
@@ -581,8 +608,7 @@ func (packet *SnmpPacket) marshalPDU() ([]byte, error) {
 	switch packet.PDUType {
 	case GetBulkRequest:
 		// requestid
-		buf.Write([]byte{2, 4})
-		err := binary.Write(buf, binary.BigEndian, packet.RequestID)
+		err := shrinkAndWriteUint(buf, int(packet.RequestID))
 		if err != nil {
 			return nil, err
 		}
@@ -620,10 +646,9 @@ func (packet *SnmpPacket) marshalPDU() ([]byte, error) {
 
 	default:
 		// requestid
-		buf.Write([]byte{2, 4})
-		err := binary.Write(buf, binary.BigEndian, packet.RequestID)
+		err := shrinkAndWriteUint(buf, int(packet.RequestID))
 		if err != nil {
-			return nil, fmt.Errorf("unable to marshal OID: %w", err)
+			return nil, err
 		}
 
 		// error status
@@ -914,17 +939,10 @@ func marshalVarbind(pdu *SnmpPDU) ([]byte, error) {
 		pduBuf.Write(tmpBuf.Bytes())
 
 	case Counter64:
-		converters := map[Asn1BER]func(interface{}) ([]byte, error){
-			Counter64: marshalUint64,
-		}
 		tmpBuf.Write([]byte{byte(ObjectIdentifier), byte(len(oid))})
 		tmpBuf.Write(oid)
 		tmpBuf.WriteByte(byte(pdu.Type))
-		intBytes, err := converters[pdu.Type](pdu.Value)
-		if err != nil {
-			return nil, fmt.Errorf("error converting PDU value type %v to %v: %w", pdu.Value, pdu.Type, err)
-		}
-
+		intBytes := marshalUint64(pdu.Value)
 		tmpBuf.WriteByte(byte(len(intBytes)))
 		tmpBuf.Write(intBytes)
 		tmpBytes := tmpBuf.Bytes()
@@ -995,7 +1013,7 @@ func (x *GoSNMP) unmarshalHeader(packet []byte, response *SnmpPacket) (int, erro
 	}
 
 	cursor += count
-	if cursor > len(packet) {
+	if cursor >= len(packet) {
 		return 0, fmt.Errorf("error parsing SNMP packet, packet length %d cursor %d", len(packet), cursor)
 	}
 
@@ -1117,8 +1135,8 @@ func (x *GoSNMP) unmarshalResponse(packet []byte, response *SnmpPacket) error {
 			return fmt.Errorf("error parsing SNMP packet, packet length %d cursor %d", len(packet), cursor)
 		}
 
-		if maxRepetitions, ok := rawMaxRepetitions.(uint32); ok {
-			response.MaxRepetitions = (maxRepetitions & 0x7FFFFFFF)
+		if maxRepetitions, ok := rawMaxRepetitions.(int); ok {
+			response.MaxRepetitions = uint32(maxRepetitions & 0x7FFFFFFF)
 		}
 	} else {
 		// Parse Error-Status
@@ -1352,3 +1370,12 @@ func (x *GoSNMP) receive() ([]byte, error) {
 	copy(resp, x.rxBuf[:n])
 	return resp, nil
 }
+
+func shrinkAndWriteUint(buf io.Writer, in int) error {
+	out, err := asn1.Marshal(in)
+	if err != nil {
+		return err
+	}
+	_, err = buf.Write(out)
+	return err
+}
